package polyglot.ext.ibex.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.types.*;
import polyglot.ext.ibex.types.*;
import polyglot.ext.ibex.visit.*;
import polyglot.visit.*;
import polyglot.util.*;
import java.util.*;

/**
 * Node representing a semantic action.
 */
public class SymbolNode_c extends RhsElement_c implements SymbolNode {
    String name;
    Symbol sym;

    public SymbolNode_c(Position pos, String name) {
        super(pos);
        this.name = name;
    }

    public String name() { return name; }
    public SymbolNode name(String name) {
        SymbolNode_c n = (SymbolNode_c) copy();
        n.name = name;
        return n;
    }

    public Symbol symbolInstance() { return sym; }
    public SymbolNode symbolInstance(Symbol sym) {
        SymbolNode_c n = (SymbolNode_c) copy();
        n.sym = sym;
        return n;
    }

    public Node buildTypes(TypeBuilder tb) throws SemanticException {
        IbexTypeSystem ts = (IbexTypeSystem) tb.typeSystem();
        Symbol sym = ts.terminal(position(), (ParserType) tb.currentClass(),
                                 Flags.PUBLIC.Final().Static(),
                                 ts.unknownType(position()), name);
        return symbolInstance(sym);
    }

    public Node disambiguate(AmbiguityRemover ar) throws SemanticException {
        IbexTypeSystem ts = (IbexTypeSystem) ar.typeSystem();
        IbexNodeFactory nf = (IbexNodeFactory) ar.nodeFactory();
        Context c = (Context) ar.context();

        if (ar.kind() != AmbiguityRemover.ALL) {
            return this;
        }

        ParserType pt = (ParserType) c.currentClass();

        Symbol sym = ts.findSymbol(pt, name);
        return symbolInstance(sym);
    }

    public Node rewrite(Rewriter rw) {
        return this;
    }
}
