
package polyj.tests;
import java.io.PrintStream;
import java.util.Stack;
import java.util.Enumeration;

public class Test {
  protected boolean printSuccesses;
  protected PrintStream out;
  protected Stack divisions;
  protected int nSucceeded; // How many have succeeded
  protected int nFailed;    // How many have failed?
  protected int depth;      // Current depth of tests.

  // Has the division changed since the last failure?
  protected boolean divisionChanged;  

  // Summarize up to which depth?
  protected int printToDepth;
  protected int[] nFailuresAtDepth;
  protected int[] nSuccessesAtDepth;

  public boolean dieOnFailure;

  public Test(String[] args) {
    this.printSuccesses = false;
    this.out = System.out;
    this.printToDepth = -1;
    divisions = new Stack();
    nSucceeded = nFailed = depth = 0;
    dieOnFailure = false;
    divisionChanged = true;
    for (int i = 0; i < args.length; ++i) {
      if (args[i].equals("-v")) {
    printSuccesses = true;
      } else if (args[i].equals("-d")) {
    if (++i == args.length) {
      System.err.println("No argument for '-d' switch.");
      System.exit(-1);
    }
    printToDepth = Integer.valueOf(args[i]).intValue();
      } else {
    System.err.println("Unrecognized argument: " + args[i]);
    System.exit(-1);
      }      
    }  
    if (printSuccesses) printToDepth=-1;
    nSuccessesAtDepth = new int[printToDepth+1];
    nFailuresAtDepth = new int[printToDepth+1];
  }
  
  public Test(boolean printSuccesses, PrintStream out, 
          int printToDepth) {
    this.printSuccesses = printSuccesses;
    this.out = out;
    this.printToDepth = printToDepth;
    divisions = new Stack();
    nSucceeded = nFailed = depth = 0;
    divisionChanged = true;
    if (printSuccesses) printToDepth=-1;
    nSuccessesAtDepth = new int[printToDepth+1];
    nFailuresAtDepth = new int[printToDepth+1];
  }

  public Test(boolean printSuccesses) {
    this(printSuccesses, System.out, -1);
  }

  public Test() {
    this(false, System.out, -1);
  }
  
  public void StartDivision(String s) {
    if (printSuccesses || depth <= printToDepth-1) {
      indent(depth);
      out.println(s + ":");
    }
    depth++;
    divisions.push((Object) s);    
  }

  public void EndDivision() {    
    if (depth <= printToDepth) {
      indent(depth);
      System.out.println(" (Passed " + nSuccessesAtDepth[depth]
             + ", Failed " + nFailuresAtDepth[depth] +")");
      nSuccessesAtDepth[depth] = 0;
      nFailuresAtDepth[depth] = 0;
    }    
    if (depth > 0) {
      divisions.pop();
      depth--;
    }
    divisionChanged = true;
  }

  public void NewDivision(String s) {
    EndDivision();
    StartDivision(s);
  }

  public void Test(String testName, boolean success, String msg) {
    // Print the result.
    if (msg.length() > 0) msg = ": " + msg;
    if (printSuccesses) {
      indent(depth);
      out.println(testName + ": " + 
          (success ? "OK" : "Failed" +msg ));       
    } else if (!success) {
      if (divisionChanged) {
    divisionChanged = false;
    Enumeration e = divisions.elements();
    while(e.hasMoreElements()) {
      out.print(((String) e.nextElement()) + "/");
    }
    out.println("");
      }
      out.println("   " + testName + ": Failed"+msg);
    }
    if (success) {
      nSucceeded++;
      for (int i = 0; i <=depth && i <=printToDepth; ++i) {
    nSuccessesAtDepth[i]++;
      }
    } else {
      nFailed++;      
      for (int i = 0; i <=depth && i <=printToDepth; ++i) {
    nFailuresAtDepth[i]++;
      }
      if (dieOnFailure)
    throw new RuntimeException();
    }
  }

  public void Test(String testName, boolean success) {
    Test(testName, success, "");
  }

  public void Conclude() {
    out.println("Results: " + nSucceeded + " ok; " + nFailed + " Failed.");
    if (nFailed == 0) 
      out.println("PASSED ALL TESTS.");
    else
      out.println("DID NOT PASS ALL TESTS.");
  }
  
  private void indent(int d) {
    for(int i = 0; i < d; ++i)
      out.print("  ");
  }

  public static void Assert(boolean success, String description) {
    if (!success)
      throw new FailedAssertionError(description);
  }
 
  public static void Assert(boolean success) {
    if (!success)
      throw new FailedAssertionError();      
  }

}

class FailedAssertionError extends Error {
   public FailedAssertionError(String x) { super(x); }
   public FailedAssertionError() { super(); }
}
